from StringIO import StringIO

# Zope
from AccessControl import ClassSecurityInfo

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

# AT & LinguaPlone support?
try:
   from Products.LinguaPlone.public import *
except ImportError:
   from Products.Archetypes.public import *

from Products.Archetypes.interfaces.base import *

#ATCT
try:
   from Products.ATContentTypes.content.folder import ATFolder #,ATBTreeFolder
   from Products.ATContentTypes.content.schemata import finalizeATCTSchema
# old ATCT
except ImportError:
   from Products.ATContentTypes.types.ATFolder import ATFolder
   #from Products.ATContentTypes.types.ATFolder import ATBTreeFolder
   finalizeATCTSchema = lambda x: None

# Third-party
from Products.ATSchemaEditorNG.ParentManagedSchema import ParentManagedSchema
from Products.ATSchemaEditorNG.SchemaEditor import SchemaEditor
from Products.ATReferenceBrowserWidget import ATReferenceBrowserWidget

# Package
from Products.eCampaigning.config import AccessEcampaigningDataPermission #, VERSION
from Products.eCampaigning import util

from record import *
from schemas import ActionSchema, ActionEditionSchema, ActionRecordSchema

__all__ = ["CampaignAction","CampaignActionEdition"]


class CampaignAction(ATFolder):
   "campaign action content"

   typeDescription= 'Parent container for different action related elements.'
   typeDescMsgId  = 'description_edit_campaignaction'

   security = ClassSecurityInfo()

   meta_type = portal_type = archetype_name = "Campaign Action"
   global_allow = 1
   content_icon = "campaignaction.png"

   _at_rename_after_creation = True

   default_view = "base_view"

   allowed_content_types = [
            "Person Target",
            "Organization Target",
            "Action Edition",
            "e-Petition Action Edition",
            "e-Message Action Edition",
            "Thank You Page Template",
            "Thank You Email Template",
            "Action Message Template",
            "I18NFolder",
            "Image",
            "Document",
            "File",
            "ATImage",
            "ATDocument",
            "ATFile"
   ]

   schema = ATFolder.schema.copy() + Schema(ActionSchema)
   
   tw = schema["title"]
   tw.label="Headline"
   tw.description = "Short headline used for 'call to action'"
   tw.label_msgid="label_action_headline"
   tw.description_msgid="help_action_headline"
   tw.i18n_domain="ecampaigning"

   schema['description'].schemata = 'default'

   try:
      schema.moveField("name", before="title")
   except AttributeError:
      #from Products.eCampaigning. import moveFieldInSchema
      util.moveFieldInSchema(schema, "name",1)

   finalizeATCTSchema(schema)

   actions = (

         {
          'id': 'metadata',
          'name': 'Properties',
          'action': 'string:${object_url}/metadata_edit_form',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
         },

        { 'id': 'getdata',
          'name': 'Get data',
          'action': 'string:${object_url}/datafile',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        },

        { 'id': 'update',
          'name': 'Check for updates',
          'action': 'string:${object_url}/updates_check',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        }

   )


   def getRecordCount(self,rtypes=("Action Record",),ctypes=("Action Edition",)):
      "common for actions & editions"
      count = 0
      for container in self.objectValues(ctypes):
         count += len(container.records.objectIds())
      return count


   def getRecords(self,rtypes=("Action Record",),ctypes=("Action Edition",)):
      "used by dataexport - get all records from all editions"
      for container in self.objectValues(ctypes):
         # use the generator
         for r in container.getRecords(rtypes):
            yield r

registerType(CampaignAction)


class CampaignActionEdition(ATFolder, SchemaEditor):
   "base class for campaign action editions"

   typeDescription= 'Configurable e-action containing steps and other action elements.'
   typeDescMsgId  = 'description_edit_campaignactionedition'

   security = ClassSecurityInfo()
   action_record = "Action Record"
   global_allow = 1
   filter_content_types = 1
   meta_type = portal_type = archetype_name = "Action Edition"
   content_icon = "actionedition.png"
   default_view = "base_view"

   # support variable for detecting type changes
   _previous_actiontype = None
   
   __implements__ = ATFolder.__implements__ + SchemaEditor.__implements__
   
   allowed_content_types = [
         "I18NFolder",
         "Person Target",
         "Organization Target",
         "Thank You Page Template",
         "Thank You Email Template",
         "Action Message Template",
         "Action Step",
         "Large Plone Folder",
         "I18NFolder",
         "Image",
         "Document",
         "File",
         "ATImage",
         "ATDocument",
         "ATFile"
   ]

   schema = ATFolder.schema.copy() + Schema(ActionEditionSchema)

   # Further schema customization
   schema['description'].schemata = 'default'

   from Products.eCampaigning.util import moveFieldInSchema

   try:
      schema.moveField("name", before="title")
   except AttributeError:
      from Products.eCampaigning.util import moveFieldInSchema
      moveFieldInSchema(schema, "name",1)

   schema['language'].default_method = "getDefaultLanguage"

   finalizeATCTSchema(schema)

   actions = (
        
        # Note: signupprocess comes from self
        { 'id': 'takeaction',
          'name': 'Take action',
          'action': "string:${object_url}/takeaction",
          'permissions': (View,),
          'category': 'object'
        },

        { 'id': 'editschemas',
          'name': 'Edit Schemas',
          'action': 'string:$object_url/atse_editor',
          'permissions': (ModifyPortalContent,),
          'category': 'object'
        },

        { 'id': 'metadata',
          'name': 'Properties',
          'action': 'string:${object_url}/metadata_edit_form',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        },

        { 'id': 'getdata',
          'name': 'Get data',
          'action': 'string:${object_url}/records/datafile',
          'permissions': (ModifyPortalContent,),
          'category': 'object',
          'visible':True
        }
   )


   if not hasattr(ATFolder, "__browser_default__"):
      from Products.PageTemplates.PageTemplateFile import PageTemplateFile
      from Products.eCampaigning.config import PACKAGE_HOME
      import os
      ptpth = os.sep.join((PACKAGE_HOME,)+ tuple("skins/ecampaigning_templates/eaction_edition_view.pt".split('/')))
      index_html = PageTemplateFile(ptpth, globals())


   def normalizeName(s):
      ""
      return s.replace(' ','_').lower()


   def canSetDefaultPage(self):
      "no need"
      return False
      

   def configureAction(self, preset_id):
      "configure edition using a preset script"
      
      self._previous_actiontype = self.getActiontype()
               
      if preset_id == "custom":
         return

      configscript = getattr(self, "configure_" + preset_id, None)
      if configscript:
         step_ids = self.objectIds("Action Step")
         self.manage_delObjects(ids=step_ids)
         configscript()
      else:
         self.restrictedTraverse("view?portal_status_message=no configuration script for %s" % preset_id)

         
   def at_post_create_script(self):
      "add record store, register & customize schema, configure action, hookup content & setup permissions"
      
      self.manage_addProduct["BTreeFolder2"].manage_addBTreeFolder(id="records", title="Action Records")      
      
      # do not show default or metadata fieldset
      #self.atse_registerObject(ActionRecord, ('default', 'metadata','message'))
      schema = Schema(ActionRecordSchema)
      self.atse_registerSchema("Action Record",  schema, domain='ecampaigning')

      actiontype = self.getActiontype()      
      if actiontype != self._previous_actiontype:
         self.configureAction(actiontype)
         
      self.hookupReferencedContent()   
      self.manage_permission("ATSE: Manage schema", ('Owner',), acquire=1, REQUEST=self.REQUEST)
  
      
   def at_post_edit_script(self):
      "configure edition based on new action type selection"
      
      actiontype = self.getActiontype()
      if actiontype != self._previous_actiontype:
         self.configureAction(actiontype)

         
   def hookupReferencedContent(self):
      "attempt to hook up templates, targets etc."
      try:
         self.setMessagetemplate(getattr(self,"default-action-message").UID())
      except:
         pass

      try:
         self.setThankyouemail(getattr(self,"default-thankyou-email").UID())
      except:
         pass

      try:
         self.setThankyoupage(getattr(self, "default-thankyou-page").UID())
      except:
         pass

      try:
         self.setTargets((getattr(self,"default-person-target").UID(),))
      except:
         pass

      if self.aq_parent.portal_type == "Campaign Action":
         self.setParentaction(self.aq_parent.UID())
         
      
   def getRecordFieldNames(context, managed_only=True):
      "ALL names"
      schema = context.atse_getSchemaById("Action Record")
      all_fields = schema.keys()
      unmanaged_fields = context.atse_getUnmanagedFieldNames("Action Record")
      if managed_only:
         return [fn for fn in all_fields if fn not in unmanaged_fields]
      return all
    
   def schemasVocabulary(self):
      return tuple([(s,s) for s in self._schemas.keys()])

   def schemataVocabulary(self, schemaname):
      return tuple([(s,s) for s in self._schemas[schemaname].keys()])

   def fieldVocabulary(self,schemaname):
      return tuple([(f,f) for f in self._schemas[schemaname].keys()])

   # TODO: remove if moving to util causes no ill effects   
   #def getElementsForLayout(self, layoutname):
   #   "get a list of elements that are used in a given layout"
   #   props = getToolByName(self,"portal_properties")["ecampaigning_properties"].layout_vocabulary
   #   data = [l for l in props if layoutname in l]
   #   if data:
   #      return data[0].strip().split('|')[1].split(':')[1].split('+')
   #   return ()
      

   def layoutVocabulary(self):
      "get the layout vocabulary"
      props = getToolByName(self,"portal_properties")["ecampaigning_properties"].layout_vocabulary
      return tuple([(l.split('|')[1].split(':')[0], l.split('|')[0]) for l in props])
      

   def getVocabularyItem(self, vocabulary_id, item_id):
      "get a single item"
      props = getattr(getToolByName(self,"portal_properties")["ecampaigning_properties"], vocabulary_id)
      item = [l.strip().split('|') for l in props if item_id in l][0]
      return tuple(item)


   def validatorVocabulary(self):
      "get the validator vocabulary for selection upon step creation"
      props = getToolByName(self,"portal_properties")["ecampaigning_properties"].step_validator_vocabulary
      v = [l.strip().split('|') for l in props]
      return tuple([(id, label) for label, id in v])


   def taskVocabulary(self):
      "get the task vocabulary for selection upon step creation"
      props = getToolByName(self,"portal_properties")["ecampaigning_properties"].step_task_vocabulary
      v = [l.strip().split('|') for l in props]
      return tuple([(id, label) for label, id in v])

     
   def getLayoutDefinition(self,layoutname):
      "middle/left/right etc"

      defined_layouts = dict([(o.getLayoutelement(),o.getId()) for o in self.objectValues("Layout Element Definition")])

      if layoutname in defined_layouts:
         return getattr(self,defined_layouts[layoutname])
      else:
         return None

         
   # TODO: convert all direct references to use getRecordContainer instead
   def getRecordContainer(self):
      "get the container object"
      return self.records


   def getRecordCount(self,rtypes=("Action Record",)):
      "common for actions & editions"
      return len(self.records.objectIds(rtypes))


   def getRecords(self,rtypes=("Action Record",)):
      "used by data export, common for actions & editions"
      return self.records.objectValues(rtypes)

   def getDefaultLanguage(self):
      "if not set, get the portal language"
      return getToolByName(self,"portal_properties")["site_properties"].default_language


   def getActionTypesVocabulary(self):
      "get a DisplayList for selecting an Action Type"
      atypedata = getToolByName(self,"portal_properties")["ecampaigning_properties"].action_types
      return DisplayList([d.split('|') for d in atypedata])

   
   #TODO: remove old getActionTypesVocabulary
   #def getActionTypesVocabulary(self):
   #   "get a DisplayList for selecting an Action Type"
   #   types = zip(map(lambda x: x.lower().replace("-","_"), ACTIONTYPES), ACTIONTYPES)
   #   return DisplayList(types)

      
   # SHORTCUTS TO REFERENCED CONTENT
   
   def getThanksPageText(self):
      "get text content of the thank-you page template"
      try:
         return self.getThankyoupage().getText()
      except:
         return ""

   def getThanksPageSubject(self):
      "get subject of the thank-you page template"
      try:
         return self.getThankyoupage().getMessagetitle()
      except:
         return ""
         
   def getThanksEmailText(self):
      "get text content of the thank-you email template"
      try:
         return self.getThankyouemail().getText()
      except:
         return ""
   
   def getMessageText(self):
      "get message template text content"
      try:
         return self.getMessagetemplate().getText()
      except:
         return ""

   def getMessageSubject(self):
      "get message template subject"
      try:
         return self.getMessagetemplate().getMessagetitle()
      except:
         pass

   def addStep(self, id, title):
      "add an action step"
      new_id = self.invokeFactory(type_name="Action Step", id=id)
      id = new_id or id
      step = getattr(self, id)
      step.setTitle(title)
      step.reindexObject(idxs=["Title"])
      return step
      
registerType(CampaignActionEdition)
