from types import UnicodeType, StringType

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent, setDefaultRoles

__all__ = ["ActionRecord", "addRecord", "addRecordForm"]

class ActionRecord(SimpleItem, PropertyManager):
   "public action content"

   meta_type = "Action Record"
   content_icon   = "action.png"

   manage_options = PropertyManager.manage_options

   __ac_permissions = (

      ('Manage properties', 
        ('manage_addProperty',
         'manage_editProperties',
         'manage_delProperties',
         'manage_changeProperties'
        ),
        ("Manager",),
      ),

      ('Access contents information',
         ('hasProperty', 'propertyIds', 'propertyValues','propertyItems',
          'getProperty', 'getPropertyType', 'propertyMap', 'propertyLabel',
          'propdict', 'valid_property_id', ''
         ),
         ('Manager',),
      )
   )


   #setDefaultRoles('View', ('Manager', 'Owner'))
   #setDefaultRoles('Access contents information',('Manager','Owner'))

   def getFullName(self):
      "get givenname + family name"
      return getattr(self, "givenname", "") + " " +  getattr(self, "familyname", "")

   def getMessage(self):
      "get message body from self or the parent action edition"
      return getattr(self, "message", self.getMessagetemplate().getText())

   def getSubject(self):
      "get message subject from self or the parent action edition"
      return getattr(self, "messagesubject", self.getMessagetemplate().getMessagetitle())


   def dataToString(self, data):
      "represent objects as strings"
      if data.__class__.__name__=="DateTime":
         value = data.strftime("%y%m%d %H%M")
      elif type(data) in (list, tuple):
         value = "\n".join(data)
      else:
         value = str(data)      
      return value


   def dataToUnicode(self, data, decoders):
      "objects to unicode"

      if decoders:
         converter = self.decodeToUnicode
      else:
         converter = lambda data, decoders: unicode(data)

      if data.__class__.__name__=="DateTime":
         value = data.strftime(u'%y%m%d %H%M')
      elif type(data) in (list, tuple):
         value = converter('\n'.join(data), decoders)
      else:
         value = converter(data, decoders)
      return value


   def decodeToUnicode(self, value, decoders):
      "try decoding the string using all decoders"
      for d in decoders:
         try:
            return v.decode(d)
         except:
            pass
      raise Exception("could not decode to unicode!")


   def replaceOrEscape(self, value, textmarker, replacechar, escchar):
      "do stuff for csv output"
      if not escchar:
         # replace textmarkers
         value = value.replace(textmarker, replacechar)
      else:
         #escape the textmarkers
         value = value.replace(textmarker, escchar+textmarker)
      return textmarker + value + textmarker


   #security.declareProtected(AccessEcampaigningDataPermission,"getFieldsAsText")
   def getFieldsAsText(self, fnames, separator=',', textmarker='"', replacechar="'", escchar=None, decoders=None):
      "return raw csv-formatted text, performing replacement or escaping of text marker found inside content"
      
      # First, get values from record
      values = [getattr(self, fn, '') for fn in fnames]
      
      try:
         values = [self.dataToUnicode(v, decoders) for v in values]
      except:
         values = [self.dataToString(v) for v in values]

      # move on, whatever the case
      data = []
      for value in values:

         if value is None:
            data.append(textmarker*2)
            continue
            
         # replace textmarkers in value with textmarker+escapechar if given, else replacechar
         data.append(self.replaceOrEscape(value, textmarker, replacechar, escchar))
      
      return separator.join(data)


   def _getEntryURLParameters(self):
      "parse entry URL into dict"
      try:
         query = self.entry_url().split('?')[1]
         pairs = query.split('&')
         return dict( [pair.split('=') for pair in query.split('&') if len(pair.split('='))==2] )
         
      except IndexError:
         return ()

   
   def manage_afterAdd(self, item, container):
      "do smthing?"
      SimpleItem.manage_afterAdd(self, item, container)

InitializeClass(ActionRecord)


def addRecord(dispatcher, id):
   "constructor"
   record = ActionRecord(id)
   record._setId(id)
   dispatcher.Destination()._setObject(id, record)


def addRecordForm():
   "form constructor"
