from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Acquisition import ImplicitAcquisitionWrapper

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

# AT & LinguaPlone support?
try:
   from Products.LinguaPlone.public import *
except ImportError:
   from Products.Archetypes.public import *

#ATCT
try:
   from Products.ATContentTypes.content.document import ATDocument
   from Products.ATContentTypes.content.folder import ATFolder   
   from Products.ATContentTypes.content.schemata import finalizeATCTSchema
   from Products.ATContentTypes.content.base import ATCTContent
# old ATCT
except ImportError:
   from Products.ATContentTypes.types.ATDocument import ATDocument
   from Products.ATContentTypes.types.ATFolder import ATFolder
   from Products.ATContentTypes.ATContentType import ATCTContent
   finalizeATCTSchema = lambda x: None


# package
from Products.eCampaigning import util


__all__ = ["ActionStep"]


class ActionStep(ATFolder):
   "Defines what to show in a given step and what to do after it"

   typeDescription= 'Defines a customizable page, validates any input and performs selected post-processing tasks.'
   typeDescMsgId  = 'description_edit_actionstep'

   meta_type = portal_type = archetype_name = "Action Step"
   content_icon = "actionstep_icon.gif"
   filter_content_types = 1
   allowed_content_types = ["Layout Element",]
   global_allow = 0
   allow_discussion = 0
   default_view = "action_step_render"

   # support variable for detecting steptype changes
   _layout_changed = False
   
   actions = (

        { 'id': 'view',
          'name': 'View',
          'action': "string:${object_url}/action_step_render",
          'permissions': (View,),
          'category': 'object'
        },
   )

   schema = ATFolder.schema.copy() + Schema((
      StringField('step_layout',
         default="twocolumn",
         mutator = "setStep_layout",
         vocabulary = "layoutVocabulary",
         widget = SelectionWidget(
            label="Layout",
            description="Select a layout for the page displayed in this step",
            label_msgid="label_actionstep_layout",
            description_msgid="help_actionstep_layout",
            i18n_domain="ecampaigning"         
         ),
      ),
      
      LinesField('step_validators',
         vocabulary="validatorVocabulary",
         multiValued=1,
         widget = InAndOutWidget(
            label="Validators",
            description="Select one or more predefined validators to apply after this step",
            label_msgid="label_actionstep_validators",
            description_msgid="help_actionstep_validators",
            i18n_domain="ecampaigning"
         ),
      ),

      LinesField('step_tasks',
         vocabulary="taskVocabulary",
         multiValued=1,
         widget = InAndOutWidget(
            label="Tasks",
            description="Select one or more predefined tasks to perform after this step",
            label_msgid="label_actionstep_tasks",
            description_msgid="help_actionstep_tasks",
            i18n_domain="ecampaigning"         
         ),
      ),

      StringField('step_submitlabel',
         schemata = "advanced",
         widget = StringWidget(
            label="Submit button label",
            description="Enter the label of the form submit button for this step.",
            label_msgid="label_step_submitlabel",
            description_msgid="help_step_submitlabel",
            i18n_domain="ecampaigning"
         ),
      ),
      
      StringField('step_custom_validator',
         schemata = "advanced",
         widget = StringWidget(
            label="Custom validator",
            description="Enter the name of a custom validator script to use",
            label_msgid="label_actionstep_customvalidator",
            description_msgid="help_actionstep_customvalidator",
            i18n_domain="ecampaigning"
         ),
      ),

      StringField('step_custom_task',
         schemata = "advanced",
         widget = StringWidget(
            label="Custom task",
            description="Enter the name of a custom task script to use",
            label_msgid="label_actionstep_customtask",
            description_msgid="help_actionstep_customtask",
            i18n_domain="ecampaigning"
         ),
      ),

   ))

   finalizeATCTSchema(schema)

   # custom mutator
   def setStep_layout(self, value, **kw):
      if value != self.getStep_layout():
         self._layout_changed = True      
      self.getField('step_layout').set(self, value, **kw)


   def getVisibleFields(self):
      "return a list of names of all fields used in this step"
      fieldblocks = []
      # go through all layout definitions
      for e in self.objectValues():
         fieldblocks += [b for b in e.objectValues("Field Block") if b.isVisible()]
      return fieldblocks


   def getFieldsToValidate(self):
      "get a list of fields to validate"
      
      allfields =  [f.getFieldname() for f in self.getVisibleFields()]
      
      if not self.getAction_message_modifiable():
         try:
            allfields.remove("message")
         except:
            pass
         try:
            allfields.remove("messagesubject")
         except:
            pass
            
      return allfields


   # ORDERED FOLDER - BASED STEP SEQUENCE UTILITIES
   
   def getNextStepId(self):
      "return id of next step"
      parent = self.aq_parent
      mypos = parent.getObjectPosition(self.getId())
      stepids = dict([(parent.getObjectPosition(id), id) for id in parent.objectIds()])
      return stepids[mypos + 1]
      
      
   def isLastStep(self):
      "return true if I am last step"
      if self.aq_parent.getObjectPosition(self.getId()) == (len(self.aq_parent.objectIds())-1):
         return True
      return False


   # STEP CONFIGURATION
      
   def addLayoutElement(self, ename):
      "add a layout definition"
      id = ename + "_def"
      new_id = self.invokeFactory(type_name="Layout Element", id=id)
      new_id = new_id or id
      e = getattr(self, new_id)
      e.setDatasource("Action Record")
      e.setLayoutelement(ename)
      e.reindexObject(idxs=["Title"])
      return new_id
      
   def addLayoutElements(self, steptype):
      "configure the layout, validators and tasks for this step"
      data = getToolByName(self, "portal_properties")["ecampaigning_properties"].layout_vocabulary
      elements = util.decode_encodedvocabulary(data, steptype)
      for epos, elementname in enumerate(elements):
         element_id = self.addLayoutElement(elementname)
         self.moveObjectToPosition(element_id, epos)
         
   
   def build_layout(self, layout):
      for epos, (elementname, blocks) in enumerate(layout):
         element_id = self.addLayoutElement(elementname)
         self.moveObjectToPosition(element_id, epos)
         element = getattr(self, element_id)
         for bpos, block in enumerate(blocks):
            
            try:
               btype, bcontent = block
               block_id = element.addBlock(btype, bcontent)
            except ValueError:
               btype, bcontent, bcondition = block
               block_id = element.addBlock(btype, bcontent, condition=bcondition)
            
            element.moveObjectToPosition(block_id, bpos)

   # TODO: figure out why this is called after step edit??? because of new child content is created????? is there a creation_flag param or not?
   def at_post_create_script(self):
      "configure step"
      if self._layout_changed:
         step_ids = self.objectIds("Layout Element")
         self.manage_delObjects(ids=step_ids)
         self.addLayoutElements(self.getStep_layout())        
      
   # TODO: if the above is used upon step edit, is this needed at all?
   def at_post_edit_script(self):
      "configure step based on new layout type selection"      
      if self._layout_changed:
         step_ids = self.objectIds("Layout Element")
         self.manage_delObjects(ids=step_ids)
         self.addLayoutElements(self.getStep_layout())
         

registerType(ActionStep)

