# python
import sys, base64, logging
from email.Header import Header
from email.Message import Message

from Products.PageTemplates.Expressions import getEngine
from Acquisition import ImplicitAcquisitionWrapper

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import mapply
from Products.Archetypes.Field import *

# package
from content.record import ActionRecord
from config import CUSTOM_DATAMERGE_SCRIPT

_marker = []


def evaluateTales(context, expressionstring):
   "evaluate TALES expression string in context"
   compiled = getEngine().compile(expressionstring)
   result = compiled(context)
   if isinstance(result, Exception):
      raise result
   return result


#TODO: check if expression caching should be added
def getActionExpressionContext(context):
   "get TALES expression context for use in action step sequence" 
   ptool = getToolByName(context, 'portal_url')
   portal = ptool.getPortalObject()
   request = context.REQUEST
   session = request.SESSION
   edition = context.getRecordContainer().aq_parent
   action = edition.getParentaction()

   try:
      record = context.restrictedTraverse(session["record_path"])
   except:
      record = None
      
   data = {
      "portal":portal,
      "action":action,
      "edition":edition,
      "record":record,
      "request":request,
      "session":session   
   }
   
   return getEngine().getContext(data)

   
def persistActionFormContents(context, form, record):
   "store contents"
  
   for field in context.atse_getSchemaById("Action Record").fields():
      widget = field.widget
      result = widget.process_form(context, field, form, empty_marker=_marker)

      if result is _marker or result is None:
         continue
   
      result = result[0]

      fname = field.getName()
      
      # if the message is not modifiable or the user has not modified the message, do not save it
      if fname == "message":
         if (not context.getAction_message_modifiable()) or (result.strip() == context.getMessagetemplate().getText().strip()):
            continue
      
      # MAP (AT) FIELD TYPES TO (ZOPE) PROPERTY TYPES
      typemap={
         "StringField": ("string", lambda x: x),
         "TextField": ("text", lambda x: x),
         "BooleanField": ("boolean", lambda x: bool(eval(x))),
         "FloatField": ("float", lambda x: x),
         "IntegerField": ("long", lambda x: x),
         "DateTimeField": ("date", lambda x: x)
      }

      ftype = field.getType().split('.')[-1]

      try:
         ptype, vfunction = typemap[ftype]
      except:
         raise Exception("unsupported record field type:%s (value:%.20s)" % (ftype, result))

      if record.hasProperty(fname):
         record.manage_changeProperties({fname:vfunction(result)})
      else:
         record.manage_addProperty(fname, vfunction(result), ptype)


# modified from Archetypes.BaseObject.validate & Schema.validate
def validate_fields(context, fields_to_validate, errors):
   "Validates the form data from the request"

   form = context.REQUEST.form
   instance = context.aq_inner

   fields = [f for f in context.atse_getSchemaById("Action Record").fields()]
   fields = [f for f in fields if f.getName() in fields_to_validate]
   fields = [f for f in fields if f.getName() not in errors]

   if fields == []:
      raise Exception("no fields to validate!")

   for field in fields:
      widget = field.widget

      # parse the form
      parse_result = widget.process_form(instance, field, form, empty_marker=_marker)
      if parse_result is None or parse_result is _marker:
         raise Exception("form validation for field %s failed!" % field.getName())
      value = parse_result[0]

      # validate the field; if it already has an error, don't add
      validation_result = field.validate(instance=instance, value=value, REQUEST=context.REQUEST, errors=errors)
      if validation_result:
         fname = field.getName()
         if fname not in errors:
            errors[fname] = validation_result

   return errors


def moveFieldInSchema(schema, fieldName, index, schemata=None):
    """Given a Schema and the name of a field in this schema, move the field
    within its schemata to the position given by index. For example, giving
    index 0 will move the field to the top of the schema. Negative indexes
    are also permitted: Giving index -1 will move it to the bottom, whilst
    -2 will make the field the second-last in its schemata.

    If the 'schemata' argument is given, this gives the name of the schemata
    the field should be placed in. If the field is in another schemata, it will
    be moved.

    The method will *not* create a copy of schema before modifying it. If you
    are moving fields from a shared schema (e.g. BaseSchema or BaseMetadata),
    you should create a copy before passing the copy in to this method, to
    avoid modifying *all* instances of that schema. Since the schema is modified
    in-place, the method does not return anything.

    May raise KeyError if fieldName cannot be found, IndexError if the index
    is out of range.
    """

    field = schema[fieldName]

    if schemata is not None:
        field.schemata = schemata
    else:
        schemata = field.schemata

    # Code adapted from ManagedSchema.moveField()

    fields = schema.fields()
    fieldNames = [f.getName() for f in fields]
    schemataNames = schema.getSchemataNames()

    schemataFields = {}
    for schemataName in schemataNames:
        schemataFields[schemataName] = schema.getSchemataFields(schemataName)

    currentSchemataFields = schemataFields[schemata]
    currentIndex = [f.getName() for f in currentSchemataFields].index(field.getName())

    if index < 0:
        index = len(currentSchemataFields) - (-1 * index)

    # Move the item by deleting it and re-inserting it in the schemata list
    del currentSchemataFields[currentIndex]
    currentSchemataFields.insert(index, field)

    schemataFields[schemata] = currentSchemataFields

    # Re-initialise the schema
    schema.__init__()
    for schemataName in schemataNames:
        for f in schemataFields[schemataName]:
            schema.addField(f)


def decode_encodedvocabulary(vocabularydata, id):
   "get list of data items stored in newline-separated title|id:data1+data2+data3... lines, by id"
   data = [l for l in vocabularydata if id in l]
   if data:
      return data[0].strip().split('|')[1].split(':')[1].split('+')
   return ()

   
def encode_smtp_header(s,encoding="utf-8"):
   "utf encoding support"
   return str(Header(s, encoding))


def getDefaultMergeData(context):
   "populate data merge namespace with global defaults"
   
   # context should be the record, if available
   
   data = {}
   
   try:
       onsite_action_count = context.getRecordCount()
   except Exception, e:
       onsite_action_count = "[could not retrieve # of actions taken: %s]" % e
       
   try:
       onsite_action_goal = context.getOnsite_action_goal()
   except Exception, e:
       onsite_action_goal = "[could not retrieve value of the on-site action goal: %s]" % e
       
   try:
       onsite_action_progress = int((float(onsite_action_count) / onsite_action_goal) * 100 + 0.5)
   except Exception, e:
       onsite_action_progress = "[could not calculate the value of action progress: %s]" % e
       
   if hasattr(context, CUSTOM_DATAMERGE_SCRIPT):
      try:
         data.update(getattr(context,CUSTOM_DATAMERGE_SCRIPT)(context))
         status = "[queried custom data provider]"
      except Exception, e:
         status = "[error in custom data provider: %s]" % e
   else:
      status = "[no custom data provider found (context: %s)]" % context.getId()

   data.update({
      "__ec_dm": "__ec_dm_success",
      "onsite_action_count":onsite_action_count,
      "onsite_action_goal":onsite_action_goal,
      "onsite_action_progress":onsite_action_progress,
      "status_custom_data_provider": status
   })
   
   return data

